﻿
using System;

public enum ValueTypes { text, number, code, boolean }

public enum RulePart { unknown, Do, As, If, end }

public class AbstractSyntaxTree
{
    public string Value;
    public AbstractSyntaxTree Left, Right;
    public int Precedence;
    public bool IsOperand { get { return Left == null && Right == null; } }
    public ValueTypes ValueType;

    public AbstractSyntaxTree(string operandValue, ValueTypes vt = ValueTypes.code)
    {
		if (operandValue == null)
			throw new ArgumentNullException("operandValue");
        Value = operandValue;
        Left = Right = null;
        Precedence = int.MaxValue;
        ValueType = vt;
    }

	public AbstractSyntaxTree(AbstractSyntaxTree original)
	{
		Value = original.Value;
		Precedence = original.Precedence;
		ValueType = original.ValueType;
		if (original.Left != null)
 			Left = new AbstractSyntaxTree(original.Left);
		if (original.Right != null)
			Right = new AbstractSyntaxTree(original.Right);
	}

	public override string ToString()
    {
        if (ValueType == ValueTypes.text && Left == null && Right == null)
            return "\"" + Value + "\"";
        if (Value == "(" && Left != null)
            return Left.Value + "(" + ((Right != null) ? Right.ToString() : "") + ")";
        if (Value == "," && Left != null && Right != null)
            return Left + ", " + Right;
        return IsOperand ? Value : "(" + (Left == null ? "LNULL" : Left.ToString()) + " " + Value + " " + (Right == null ? "RNULL" : Right.ToString()) + ")";
    }

	public bool Contains(string p)
	{
		if (Value == p) return true;
		if (Left != null && Left.Contains(p)) return true;
		if (Right != null && Right.Contains(p)) return true;
		return false;
	}

	public bool IsAssignedTo(string p)
	{
		if (Value == ":=") return (Left != null && Left.Contains(p));
		if (Left != null && Left.IsAssignedTo(p)) return true;
		if (Right != null && Right.IsAssignedTo(p)) return true;
		return false;
	}
}


