﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

class Program
{
    public static List<Rule> AllRules = new List<Rule>();
	public static List<PrecedenceRuling> PrecedenceRulings = new List<PrecedenceRuling>();
	public static Stack<Rule> LocalVariables = new Stack<Rule>();
	public static string CurrentFileScope = "TheStandardRules.txt";

    static void Main(string[] args)
    {
		if (args.Length == 0)
		{
			Console.WriteLine("Please pass the source file(s) on the command line.");
			return;
		}
	    var parser = new Parser();
		parser.Parse(File.ReadAllText(CurrentFileScope, Encoding.UTF8));
		foreach (string filename in args)
        {
			CurrentFileScope = filename;
			parser.Parse(File.ReadAllText(filename, Encoding.UTF8));
		}
        AllRules.RemoveAll(r => r.Atom == null);
        if (AllRules.All(r => r.Atom != "ProgramStart"))
            throw new Exception("Where to do first?  do ProgramStart as {...}.");
        if (AllRules.All(r => r.Atom != "StoryTitle"))
            throw new Exception("We lack a title: do StoryTitle as \"...\".");
        if (AllRules.All(r => r.Atom != "StoryAuthor"))
            throw new Exception("We lack an author: do StoryAuthor as \"...\".");

        // print rules
        string headers = "";
        foreach (Rule rule in AllRules)
        {
            if (rule.Location != headers)
            {
                headers = rule.Location;
                Console.WriteLine("\n" + headers + "\n");
            }
            Console.WriteLine(rule.Atom + ((rule.IsPrivateToFile == null) ? "" : " (See " + rule.IsPrivateToFile + ")"));
            Console.WriteLine("body: " + rule.Code);
            Console.WriteLine("when: " + rule.Condition);
            Console.WriteLine();
        }

		// two identical description-of-rules won't pass == test because the test is only reference-equality; fix that now.
		var ruledescriptions = new List<RuleDescription>();
		foreach (var prule in PrecedenceRulings)
		{
			foreach (var ruledesc in ruledescriptions)
				if (ruledesc != prule.First && ruledesc.Equals(prule.First))
				{
					prule.First = ruledesc;
					goto foundFirst;
				}
			ruledescriptions.Add(prule.First);
		foundFirst:
			foreach (var ruledesc in ruledescriptions)
				if (ruledesc != prule.Second && ruledesc.Equals(prule.Second))
				{
					prule.Second = ruledesc;
					goto foundSecond;
				}
			ruledescriptions.Add(prule.Second);
		foundSecond:
			;
		}

		// check (and print) precedence rules
	    var badPrecedenceRules = new List<PrecedenceRuling>();
	    foreach (PrecedenceRuling prule in PrecedenceRulings)
	    {
			var rulesWhichQualify1 = new List<Rule>();
			foreach (Rule rule in AllRules)
				if (prule.First.Contains(rule)) 
					rulesWhichQualify1.Add(rule);
			var rulesWhichQualify2 = new List<Rule>();
			foreach (Rule rule in AllRules)
				if (prule.Second.Contains(rule))
					rulesWhichQualify2.Add(rule);
			Console.WriteLine("The " + rulesWhichQualify1.Count + " " + prule.First + " PRECEDE the " + rulesWhichQualify2.Count + " " + prule.Second);
		    rulesWhichQualify1.RemoveAll(r => !rulesWhichQualify2.Contains(r));
		    if (rulesWhichQualify1.Count > 0)
		    {
				Console.WriteLine("ERROR: this precedence rule cannot be satisfied because the following rule(s) are in both groups:");
			    foreach (Rule rule in rulesWhichQualify1)
				    Console.WriteLine("  " + rule.Atom);
				badPrecedenceRules.Add(prule);
		    }
			Console.WriteLine();
	    }
	    PrecedenceRulings.RemoveAll(badPrecedenceRules.Contains);

		// topsort the precedence rules to look for group-level conflicts
	    var graph = new DirectedGraph { Nodes = ruledescriptions.ConvertAll(rd => new Node(rd)), };
		graph.Edges = PrecedenceRulings.ConvertAll(pr => new Edge(graph.Nodes.Find(n => n.term == pr.First), graph.Nodes.Find(n => n.term == pr.Second)));
	    var sorted = new TopologicalSort(graph);
		if (!sorted.Success) Console.WriteLine(sorted); 

	    // run program
        new Interpreter().RunProgram(AllRules);

        // print rules (and "variables")
	    foreach (Rule rule in AllRules)
		    Console.WriteLine(rule.Atom + ": " + rule.Code);

        Console.ReadKey();
    }

}

