﻿using System;

public class Rule
{
    public string Atom;
    public AbstractSyntaxTree Code;
    public AbstractSyntaxTree Condition;

    public string IsPrivateToFile;
    public string Location;

	public override string ToString()
	{
		return "Rule for " + (Atom ?? "");
	}
}

public class RuleDescription // like an Inform7 description-of-objects
{
	public string NamedLike; // for NamedLike, a * is a wildcard
	public string CheckedInTheCondition;
	public string ReadInTheBody;
	public string ChangedByTheBody;
	public string UnderThisHeading;
	private string[] NamePieces;

	public override bool Equals(object obj)
	{
		var rd2 = obj as RuleDescription;
		if (rd2==null) return base.Equals(obj); // standard is reference-equality, not contents equality
		if (NamedLike != rd2.NamedLike) return false;
		if (CheckedInTheCondition != rd2.CheckedInTheCondition) return false;
		if (ReadInTheBody != rd2.ReadInTheBody) return false;
		if (ChangedByTheBody != rd2.ChangedByTheBody) return false;
		if (UnderThisHeading != rd2.UnderThisHeading) return false;
		return true;
	}

	public bool Contains(Rule rule)
	{
		if (CheckedInTheCondition != null)
			if (rule.Condition == null || !rule.Condition.Contains(CheckedInTheCondition)) return false;
		if (ReadInTheBody != null)
			if (rule.Code == null || !rule.Code.Contains(ReadInTheBody)) return false;
		if (UnderThisHeading != null )
			if (rule.Location == null || !rule.Location.Contains(UnderThisHeading)) return false;
		if (NamedLike != null)
		{
			if (rule.Atom == null) return false;
			if (NamePieces == null)
				NamePieces = NamedLike.Split(new[] {'*'}, StringSplitOptions.RemoveEmptyEntries);
			int i = 0;
			foreach (string piece in NamePieces)
			{
				i = rule.Atom.IndexOf(piece, i);
				if (i == -1) return false;
				i += piece.Length;
			}
		}
		if (ChangedByTheBody != null)
			if (rule.Code == null || !rule.Code.IsAssignedTo(ChangedByTheBody)) return false;
		return true;
	}

	public override string ToString()
	{
		return "rules" +
			   (NamedLike != null ? " named " + NamedLike : "") +
			   (UnderThisHeading != null ? " under " + UnderThisHeading : "") +
		       ((CheckedInTheCondition ?? ReadInTheBody ?? ChangedByTheBody) == null
			       ? ""
			       : " in which" +
			         (CheckedInTheCondition != null ? " the condition contains " + CheckedInTheCondition : "") +
			         (ReadInTheBody != null ? " the body reads " + ReadInTheBody : "") +
			         (ChangedByTheBody != null ? " the body changes " + ChangedByTheBody : ""));
	}
}

public class PrecedenceRuling
{
	public RuleDescription First;
	public RuleDescription Second;

	public override string ToString()
	{
		return First + " PRECEDE " + Second;
	}
}


